#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <pthread.h>
#include <semaphore.h>
#include <memory>
#include <errlog.h>
#include "comms.h"

typedef void(*TFUNC)();

ServerSocket::AddrType serverIP;// = IP4Addr( SERVER_NAME, SERVER_PORT );

pthread_mutex_t Lock::progress = PTHREAD_MUTEX_INITIALIZER;

struct Argums {
    std::auto_ptr<Lock> ptr_lock;
    std::auto_ptr<TASK> ptr_task;
    TFUNC prt_funct;
    Argums( std::auto_ptr<Lock> pl_, std::auto_ptr<TASK> pt_, TFUNC pf_ )
           : ptr_lock(pl_), ptr_task(pt_), prt_funct(pf_) {}
};

Lock::Lock() {
    if( pthread_mutex_trylock( &progress ) == EBUSY )
         throw ErrBusy( ERR_ALG, "Already Lock is constructed" );
}

bool Lock::busy() {
    if( pthread_mutex_trylock( &progress ) == EBUSY ) return true;
    pthread_mutex_unlock( &progress );
    return false;
}

namespace {
    const std::string err_stat_ersyn(" ERROR STATUS=ERSYN");
    const std::string err_stat_parked(" ERROR STATUS=PARKED");
    const std::string err_stat_fatal(" ERROR STATUS=ERFAT");
    const std::string err_stat_busy(" ERROR STATUS=BUSY");
    const std::string ok_stat_ready(" OK STATUS=READY");
    const std::string ok_stat_parked(" OK STATUS=PARKED");
    const std::string ok_stat_busy(" OK STATUS=BUSY");
    const std::string stat_ready("STATUS=READY");
    const std::string stat_parked("STATUS=PARKED");
    const std::string stat_busy("STATUS=BUSY");
}

namespace Thread {
    pthread_t Data;
    sem_t sem;
    int retw = 0;
    void create( Argums* );
    void *thread( void* );
}

void Thread::create( Argums* ars ) {
    sem_init( &sem, 0, 0 );
    pthread_create( &Data, NULL, thread, ars );
    sem_wait( &sem );
    sem_destroy(&sem);
}

void *Thread::thread( void* a ) {
    std::string status;
    Argums ts = *(Argums*)a;
    sem_post(&sem);
    try {
        ts.prt_funct();
        status = (DEVICE::exist())? ok_stat_ready : ok_stat_parked;
    }
    catch (Error &err) {
        ErrlogSingl::instance().write( err );
        status = err_stat_fatal;
    }
    catch (const std::exception& e ) {
        ErrlogSingl::instance().write( ERR_STD_EXCP, e.what() );
        status = err_stat_fatal;
    }
    catch (...) {
        ErrlogSingl::instance().write( ERR_UKN_EXCP, " Data thread finish" );
        status = err_stat_fatal;
    }
    try {
        TASK* to = ts.ptr_task.get();
        (to->source)->sendResponse( to->id + status );
        pthread_detach( Data );
    }
    catch (const std::exception& e ) {
        ErrlogSingl::instance().write( ERR_STD_EXCP, e.what() );
    }
    return(&retw);
}

int Comm::cmd_line( int ac, char **av ) {
    opterr = 0;
    int c, errcount = 0;
    std::pair< std::string, unsigned short > server( SERVER_NAME, SERVER_PORT );

    while((c = getopt(ac, av, "hdbac:p:i:")) != -1)  {
        switch(c) {
            case 'h':   std::cerr << "Read manual, at least" << std::endl;
                        break;
            case 'd':   DEVICE::cmd.test_mode = true;
                        break;
            case 'b':   DEVICE::cmd.back_mode = true;
                        break;
            case 'a':   DEVICE::cmd.auto_init = true;
                        break;
            case 'c':   if(optarg!=NULL) DEVICE::cmd.cfgfile = program_etc+std::string( optarg );
                        break;
            case 'i':   if(optarg!=NULL) server.first = std::string( optarg );
                        break;
            case 'p':   if(optarg!=NULL) server.second = atoi( optarg );
                        break;
            default:    ErrlogSingl::instance().write( ERR_CMD_OPTION, "Command line" );
                        errcount++;
                        break;
        }
    }
    ErrlogSingl::instance().debug(DEVICE::cmd.test_mode);
    serverIP = IP4Addr( server.first, server.second );
    if( setreuid(geteuid(), geteuid()) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, "Can not set mass user " ); }
    if( setregid(getegid(), getegid()) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, "Can not set mass group" ); }
    if( access(program_etc.c_str(), F_OK|R_OK ) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, std::string(program_etc) ); }
    if( access(program_res.c_str(), F_OK|W_OK ) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, std::string(program_res) ); }
    if( access(program_log.c_str(), F_OK|W_OK ) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, std::string(program_log) ); }
    if( access(DEVICE::cmd.cfgfile.c_str(), F_OK|R_OK ) < 0 )
        { errcount++ ; ErrlogSingl::instance().write( (ErrorsCode)errno, DEVICE::cmd.cfgfile ); }
//   background
    if( DEVICE::cmd.back_mode && errcount == 0 ) {
        ErrlogSingl::instance().write( "Program demonization ... done" );
        int pid = fork();
        if (pid < 0) {
            ErrlogSingl::instance().write( (ErrorsCode)errno, "Call fork() failed " );
            exit(1);
        }
        if (pid == 0) {
            setsid();
            close( 0 ); close( 1 ); close( 2 );   // redirect stdout/stderr etc to /dev/null
            int fd = open("/dev/null", O_RDWR);
            dup( fd ); dup( fd ); dup( fd );
            errcount = 0;
        } else {
            errcount = 1;
        }
    }
    if( DEVICE::cmd.auto_init && errcount == 0 ) {
        try {
/*            std::auto_ptr<TASK> tu =  std::auto_ptr<TASK>( new TASK());
            Argums argums( std::auto_ptr<Lock>( new Lock() ), tu, (TFUNC)&DEVICE::create );
            Thread::create( &argums );*/
            DEVICE::create();
            ErrlogSingl::instance().write( "Auto-initialization done..." );
            return errcount;
        }
        catch ( Error &err) {
            ErrlogSingl::instance().write( err );
            errcount = 1;
        }
        catch (const std::exception& e ) {
            ErrlogSingl::instance().write( ERR_STD_EXCP, e.what() );
            errcount = 1;
        }
        ErrlogSingl::instance().write( " Auto-initialization failed" );
    }
    return errcount;
}

ServerSocket::AddrType& Comm::deviceServer() {
    return serverIP;
}

bool Comm::deviceWatch() {
                //  
    if( !DEVICE::works() ) {
        DEVICE::destroy();
        sleep(10);
        try {
            DEVICE::create();
            ErrlogSingl::instance().write( "Auto-restoration done..." );
            return true;
        }
        catch ( Error &err) {
            ErrlogSingl::instance().write( err );
        }
        catch (const std::exception& e ) {
            ErrlogSingl::instance().write( ERR_STD_EXCP, e.what() );
        }
        ErrlogSingl::instance().write( "Auto-restoration failed" );
        return false;
    }
    return true;
}

namespace {

    typedef std::string(*CALLF_T)(std::auto_ptr<TASK>);
    typedef std::pair< std::string, CALLF_T> cfpair;

    cfpair supp_commands[] =  {
        cfpair( "init", &Comm::init ),
        cfpair( "park", &Comm::park ),
        cfpair( "quit", &Comm::quit ),
        cfpair( "run",  &Comm::run  ),
        cfpair( "stop", &Comm::stop ),
        cfpair( "get",  &Comm::get  ),
        cfpair( "set",  &Comm::set  )
    };
}

std::string Comm::call_command( Client* from, std::string com ) {

    static std::map< std::string, CALLF_T, Cmpi > func( supp_commands, supp_commands + sizeof(supp_commands)/sizeof(supp_commands[0]));
    std::vector<std::string> inp;

    Tool::parse( com.c_str(), inp );
    if( inp.size() < 2 ) {
        ErrlogSingl::instance().write( ERR_PROT_SYNT, "No command or ID" );
        return inp.at(0)+err_stat_ersyn;
    }
    if( func.find( inp.at(1) ) == func.end() ) {
        ErrlogSingl::instance().write( ERR_PROT_WRCOMM, inp.at(0)+" "+inp.at(1) );
        return inp.at(0)+err_stat_ersyn;
    }
    std::string reply;
    std::auto_ptr<TASK> ts( new TASK( from, inp.at(0), inp.at(1), std::vector<std::string>(inp.begin()+2,inp.end()) ) );
    try {
        reply = (*func[inp.at(1)])( ts );
    }
    catch ( ErrBusy& err) {
        return inp.at(0)+err_stat_busy;
    }
    catch ( ErrSyn& err) {
        ErrlogSingl::instance().write( err );
        return inp.at(0)+err_stat_ersyn;
    }
    catch ( ErrFat& err) {
        ErrlogSingl::instance().write( err );
        return inp.at(0)+err_stat_fatal;
    }
    catch ( ErrPark& err) {
        ErrlogSingl::instance().write( err );
        return inp.at(0)+err_stat_parked;
    }
    catch ( const std::exception& err ) {
        ErrlogSingl::instance().write( err );
        return inp.at(0)+err_stat_fatal;
    }
    return reply;
}

std::string Comm::init( std::auto_ptr<TASK> ts ) {
    std::string id = ts->id;
    if( DEVICE::exist() ) return id + ok_stat_ready;
    Argums argums( std::auto_ptr<Lock>( new Lock() ), ts, (TFUNC)&DEVICE::create );
    Thread::create( &argums );
    std::ostringstream wait;
    wait <<  DEVICE::initLife();
    return id + " OK WAIT=" + wait.str();
}

void setting( std::vector<std::string>& s, std::string& k ) {
    std::string::size_type i;
    for( unsigned int j = 0; j < s.size(); j++ ) {
        if( (i = s.at(j).find("=",0)) != std::string::npos ) {
            std::string key = s.at(j).substr( 0, i );
            std::string val = s.at(j).substr( i+1 );
            DEVICE::device_ptr()->set(key,val);
            s.at(j).erase(i);
        } else k += " " + s.at(j);
    }
}

void setting( std::vector<std::string>& s ) {
    std::string t;
    setting( s, t );
}

std::string Comm::run( std::auto_ptr<TASK> ts ) {
    std::string id = ts->id;
    Argums argums( std::auto_ptr<Lock>( new Lock() ), ts, (TFUNC)&DEVICE::runproc );
    setting(argums.ptr_task.get()->params);
    std::ostringstream wait;
    wait << DEVICE::device_ptr()->specify(argums.ptr_task.get()->params);
    Thread::create( &argums );
    return id + " OK WAIT=" + wait.str();
}

std::string Comm::stop( std::auto_ptr<TASK> ts ) {
    if( !Lock::busy() ) return ts->id + ok_stat_ready;
    DEVICE::device_ptr()->stop(ts->params);
    return ts->id +ok_stat_busy;
}

std::string Comm::set( std::auto_ptr<TASK> ts ) {
    Lock lock;
    std::string monkey;
    setting( ts->params, monkey );
    if( !monkey.empty() ) throw ErrSyn( ERR_PROT_NOVAL, monkey );
    return ts->id + ok_stat_ready;
}

namespace {
    std::string get_status() {
        return (DEVICE::exist())? (Lock::busy()? stat_busy : stat_ready ) : stat_parked;
    }

    std::string get_ident() {
        std::ostringstream str;
        str << "IDENT=\"" << progname << VERSION << '"';
        return str.str();
    }

    std::string get_errmes() {
        std::string str = ErrlogSingl::instance().err_message();
        if( str == "" ) return ok_stat_ready;
        std::ostringstream ostr;
        ostr << "ERROR=\"" << str << '"';
        return ostr.str();
    }

    typedef std::string(*GETF_T)();
    typedef std::pair<std::string, GETF_T> GET_PAIR;
    GET_PAIR get_kwd[] =  {
        GET_PAIR("status", get_status),
        GET_PAIR("ident",  get_ident),
        GET_PAIR("error",  get_errmes)
    };
}

std::string Comm::get( std::auto_ptr<TASK> ts ) {
    static std::map<std::string, GETF_T, Cmpi> sunf(get_kwd, get_kwd+sizeof(get_kwd)/sizeof(get_kwd[0]));
    std::string reply(" OK") ;
    for( unsigned int j = 0; j < ts->params.size(); j++ ) {
        if(sunf.find(ts->params.at(j)) != sunf.end()) {
            reply += " " + (*sunf[ts->params.at(j)])();
        } else {
            reply += " " + DEVICE::device_ptr()->get(ts->params.at(j));
        }
    }
    return ts->id + reply;
}

std::string Comm::park( std::auto_ptr<TASK> ts ) {
    std::string id = ts->id;
    if( !DEVICE::exist() ) return ts->id + ok_stat_parked;
    Argums argums( std::auto_ptr<Lock>( new Lock() ), ts, (TFUNC)&DEVICE::destroy );
    std::ostringstream wait;
    wait <<  DEVICE::parkLife();
    Thread::create( &argums );
    return id + " OK WAIT=" + wait.str();
}

std::string Comm::quit( std::auto_ptr<TASK> ts ) {
    Client* the = ts->source;
//    if( DEVICE::exist() ) DEVICE::device_ptr()->quit( ts.get()->params );
    std::string reply = park( ts );
    the->quit();
    return reply;
}

void Comm::terminate() {
    DEVICE::destroy();
}


