#ifndef _ERRLOG_H
#define _ERRLOG_H

#include <string>
#include <fstream>
#include <iostream>

#include "singleton.h"

enum ErrorsCode {
    ERR_CFG_STRUCT = 600,
    ERR_CFG_MISSED,
    ERR_CFG_CHANNEL,
    ERR_INI_OBJECT = 610,
    ERR_NOT_CONFIGURED,
    ERR_UNKNOWN_ADDRESS,
    ERR_CENT_EMPTY = 620,
    ERR_CCD_EMPTY,
    ERR_NORM_EMPTY,
    ERR_NORM_SHIF,
    ERR_IEEE_BUS = 630,
    ERR_CAM_NOTF,
    ERR_CAM_IDEN,
    ERR_CAM_INIT,
    ERR_CAM_ERR,
    ERR_DRIVER_VERSION = 640,
    ERR_EXCHANGE,
    ERR_EXC_STAT,
    ERR_NOT_REPLY,
    ERR_IDENT,
    ERR_TEST,
//    ERR_OVRLIGHT,
//    ERR_MIRROR,
    ERR_DEV_FUNC,
    ERR_DEV_STAT,
    ERR_DEV_RANGE,
    ERR_ALG = 650,
    ERR_REG_STRUCT,
    ERR_EMPTY_MODE,
    ERR_NOTHING_DONE,
    ERR_PROT_SYNT = 660,
    ERR_PROT_WRCOMM,
    ERR_PROT_NOKWD,
    ERR_PROT_WRKWD,
    ERR_PROT_SCEN,
    ERR_PROT_NOVAL,
    ERR_SOCK_TOUT,
    ERR_SOCK_BAD,
    ERR_CMD_OPTION = 670,
    ERR_STD_EXCP,
    ERR_UKN_EXCP,
    ERR_SERIAL = 680,
    ERR_PROT_MEADE,
    LOG_ONLY = 699,
    WARNING = 0
};

class Error : public std::runtime_error {
    const ErrorsCode m_cod;
//    const std::string m_mess;
protected:
    Error( const ErrorsCode c,  const std::string& mess );
public:
//    const std::string message() const { return what();} // obsolete
    ErrorsCode err() const { return m_cod;}
};

class ErrSyn : public Error {
public:
    ErrSyn( const ErrorsCode c, const std::string& mess) : Error(c, mess) {}
};

class ErrFat : public Error {
public:
    ErrFat( const ErrorsCode c, const std::string& mess) : Error(c, mess) {}
};

class ErrPark : public Error {
public:
    ErrPark( const ErrorsCode c, const std::string& mess) : Error(c, mess) {}
};

class ErrBusy : public Error {
public:
    ErrBusy( const ErrorsCode c, const std::string& mess) : Error(c, mess) {}
};

class Errlog {
    std::fstream log;
    std::string current_log;
    std::string last_error;
    std::string last_warning;
    int repeated;
    bool testmode;
private:
    Errlog();
    Errlog(const Errlog&);
    Errlog& operator=(const Errlog&);
    ~Errlog();
    void open( const std::string );
    const char* strerror( ErrorsCode );
friend class PhoenixSingletonTraits<Errlog>;
public:
    void debug( const bool d ) { testmode = d; };
    bool isTest() { return testmode; }
    void write( const Error& );
    void write( const std::exception& );
    void write( ErrorsCode, const std::string& );
    void write( const std::string& );
    void write_if_test( const std::string& );
    void wthrow( ErrorsCode, const std::string& );
    void warning( const std::string &, double  );
    void warning( const std::string &, int   );
    void warning( const std::string & );
    const std::string& err_message() {return last_error; } 
    void reopen();
};

typedef Singleton<Errlog, PhoenixSingletonTraits> ErrlogSingl;

#endif
