#ifndef _INFRATYP_H
#define _INFRATYP_H
#include <vector>
#include <string>
#include <sstream>
#include <iomanip>
#include <math.h>

template<class T> const T& max( const T& a, const T& b) {
    return ( a < b )? b : a;
}

template<class T> const T& min( const T& a, const T& b) {
    return ( a < b )? a : b;
}

template<class T> const T& max_abs( const T& a, const T& b) {
    return ( abs(a) < abs(b) )? b : a;
}

template<class T> const T& min_abs( const T& a, const T& b) {
    return ( abs(a) < abs(b) )? a : b;
}

template<class T> T sign( const T& a ) {
    return ( a < 0 )? -1 : 1;
}

template<class T> T time24( const T& a ) {
    return ( a < 12.0 )? a + 24.0 : a;
}

template<class T> T night_time_interval( const T& a ) {
    T b = fmod( a, 24.0 );
    return ( b < 9.0 )? b + 24.0 : b;
}

template<class T> T nint( T a ) {
    return 2*M_PI*(a/(2*M_PI) - floor(a/(2*M_PI)) );
}

template<class T> T sint( T a ) {
    return nint( a + M_PI ) - M_PI;
}

template<class T> class point {
    T _x;
    T _y;
public:
    point( T u = 0, T v = 0 ): _x(u), _y(v) {}
    T  x() const { return _x; }
    T  y() const { return _y; }
    T  mean() const { return _x; }
    T  vars() const { return _y; }
    point operator+=( point a ) {
        _x += a._x;
        _y += a._y;
        return *this;
    }
    point operator-=( point a ) {
        _x -= a._x;
        _y -= a._y;
        return *this;
    }
    point operator*=( point a ) {
        _x *= a._x;
        _y *= a._y;
        return *this;
    }
    point operator/=( double a ) {
        _x /= a;
        _y /= a;
        return *this;
    }
    point operator/( double a ) {
        point t = *this;
        t /= a;
        return t;
    }
    point operator+( point a ) {
        a._x += _x;
        a._y += _y;
        return a;
    }
    point operator-( point a ) {
        a._x = _x - a._x;
        a._y = _y - a._y;
        return a;
    }
    point operator*( point a ){
        point t = *this;
        t *= a;
        return( t );
    }

    double distance( point a = point() ) {
        return sqrt( (_x-a._x)*(_x-a._x)+ (_y-a._y)*(_y-a._y) );
    }

    bool operator==( point a ){
        return _x == a._x && _y == a._y;
    }

    template<class U> operator point<U>() const {
        return point<U>(static_cast<U>(_x),static_cast<U>(_y));
    }
};

namespace std{
  template<class T> std::istream& operator>>(std::istream& stm, point<T>& p){
    T x, y;
    stm >> x >> y;
    p = point<T>( x, y );
    return stm;
  }
  template<class T> std::ostream& operator<<(std::ostream& stm, point<T>& p){
    stm << p.x() << ", " << p.y();
    return stm;
  }
};

#endif
