#include <signal.h>
#include <cmath>

#include "rs485.h"

RS485::RS485(const std::string& d, int r) : device(d), baudrate(r) {
    Lock lock;
    try {
        ptrs = std::auto_ptr<RS485impl>(new RS485impl(device, baudrate));
    }
    catch (ErrFatal& e) {
        ErrlogSingl::instance().wthrow( ERR_EXC_STAT, e.what() );
        ptrs = std::auto_ptr<BaseRS485>(new BaseRS485());
    }
    catch (std::exception& e) {
        throw ErrFat(ERR_STD_EXCP, e.what() );
    }
    if( ptrs->protocol().size() ) ErrlogSingl::instance().write( "RS485 initialization with " + ptrs->protocol() );
}

RS485::~RS485() {
        std::ostringstream statstr;
        statstr << "Packets total: " <<  ptrs->stat_packets() << " Errors total: " << ptrs->stat_errors();
        ErrlogSingl::instance().write( statstr.str() );
}

void RS485::reset() {
    ptrs.reset();
    try {
        ptrs = std::auto_ptr<RS485impl>( new RS485impl( device, baudrate) );
    }
    catch( ErrFatal& e) {
        ErrlogSingl::instance().wthrow( ERR_EXC_STAT, e.what() );
        ptrs = std::auto_ptr<BaseRS485>(new BaseRS485());
    }
    catch (std::exception& e) {
        throw ErrFat(ERR_STD_EXCP, e.what() );
    }
}

void RS485::error_handler( RS485::ErrDriver& e ) {
    std::string mess(e.what());
    if( mess == "RS driver timeout" ) throw ErrFat( ERR_EXC_STAT, e.what() );
    throw RS485::ErrSignal( mess );
}

pthread_mutex_t RS485::Lock::mtx = PTHREAD_MUTEX_INITIALIZER;

