/* The GPL applies to this program.
  In addition, as a special exception, the copyright holders give
  permission to link the code of portions of this program with the
  OpenSSL library under certain conditions as described in each
  individual source file, and distribute linked combinations
  including the two.
  You must obey the GNU General Public License in all respects
  for all of the code used other than OpenSSL.  If you modify
  file(s) with this exception, you may extend this exception to your
  version of the file(s), but you are not obligated to do so.  If you
  do not wish to do so, delete this exception statement from your
  version.  If you delete this exception statement from all source
  files in the program, then also delete it here.
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <sys/stat.h>
#include <sys/types.h>
#include <signal.h>

#include <server.h>
#include <comms.h>
#include <errlog.h>
#include "defnames.h"

bool expire = false;

void catch_fault( int sig ) {
    ErrlogSingl::instance().write( "SIGTERM catched! Observation will be finished" );
    Comm::terminate();
    expire = true;
}

int main(int ac, char **av ) {
    std::cerr << "\n\t\t" << progname << VERSION << std::endl;
    if( Comm::cmd_line( ac, av ) ) return 1;
    try {
        Server server( Comm::deviceServer(), &Comm::call_command );
        signal( SIGTERM, catch_fault );
        while( !expire && ( Lock::busy() || !server.isQuit()) ){
            try {
                SocketWatcher::instance().doPoll(3);
            }
            catch (const std::exception& e ) {
                ErrlogSingl::instance().write( std::string("Main loop: ") + e.what() );
            }
            catch (...) {
                ErrlogSingl::instance().write( "Main loop: Unknown type exception" );
            }
            if( !Comm::deviceWatch() ) {
                ErrlogSingl::instance().write( "Exit with code 1" );
                            //    !!
                return 1;
            }
        }
    }
    catch (const std::exception& e ) {
        ErrlogSingl::instance().write( std::string("Main: ") + e.what() );
    }
    std::cerr << "\n\t\t" << progname << VERSION << " finished" << std::endl;
    return 0;
}
