#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <signal.h>
#include <vector>
#include <iostream>
#include <iomanip>
#include <stdexcept>

#include <rs485.h>
#include <astrotime.h>

#include "sens_device.h"
#include "results.h"
#include <client_handler.h>

AstroDateTime utc;

const std::string   sect_device("Devices");
const std::string   subs_device("Actuators");
const std::string   subs_meteo("Conditions");
const std::string   subs_sky("Sky Sensor");
const std::string   subs_power("Power");

pthread_t Device::Data_thread;
int Device::proc_run = 0;

Device::Device( Config& cf,  bool tst ) try : device( cf.inp<std::string>( "General", "Common", "Device") ), rs485( device ) {
                    //     RS485
    meteo_station = std::auto_ptr<Meteo>( new Meteo( cf, rs485, subs_meteo ) );
    dome_controller = std::auto_ptr<Curtain>( new Curtain( cf, rs485, subs_device ) );
    sky_sensor = std::auto_ptr<SkySens>( new SkySens( cf, rs485, subs_sky ) );
//    std::cerr << "Constructor Device" << std::endl;
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_STD_EXCP, e.what() );
}

Device::~Device(void) {
}

void Device::monitor_run( double sp, double at ) {
    scan_period = sp;
    accum_time = at;
    accum_end = utc.currentSec() + accum_time;
    pthread_create( &Data_thread, NULL, &Device::thread, (void*)this );
}

void Device::monitor_stop(){
    if( pthread_cancel( Data_thread ) == ESRCH ) {
        ErrlogSingl::instance().write( "Data thread is terminated already");
        return;
    }
    if( pthread_join( Data_thread, NULL) != 0 ) {
        ErrlogSingl::instance().write(ERR_ALG, "Data thread pthread_join failed");
    }
}

void Device::monitor_result( MonitorInfo& c ) {
    out->meteoResults( c );
    accum.push_back( c );
    if( utc.currentSec() > accum_end ) {    // Calculation of the accumulated data
        MonitorInfo reduced;
        meteo_station->accumulate( accum, reduced );
        sky_sensor->accumulate( accum, reduced );
        accum_end = utc.currentSec() + accum_time;
        accum.clear();
        out->meteoAccResults( reduced );
    }
}

bool Device::monitor_watch() {
    static int run_proc_prevs = -1;
    if( run_proc_prevs == proc_run ) return false;
    run_proc_prevs = proc_run;
    return true;
}

bool Device::expire = false;

// void Device::catch_fault(int sig) {
//     expire = true;
//     ErrlogSingl::instance().write( "Meteo: Signal SIGTERM catched!" );
// }

void *Device::thread( void* a ) {
    Device* ptr = (Device*)a;
    MonitorInfo x;
//    signal( SIGTERM, ptr->catch_fault );
    while( !expire ) {
        proc_run++;
        try {
            if( sleep((int)ptr->scan_period) > 0 ) return 0;      //     scan_period
            if((ptr->meteo_station)->connection()) {
                (ptr->meteo_station)->getData( x );
            }
            if((ptr->dome_controller)->connection()) {
                (ptr->dome_controller)->getData( x );
                (ptr->sky_sensor)->getData( x );
            }
            if((ptr->sky_sensor)->connection()) {
                (ptr->sky_sensor)->getData( x );
            }
            ptr->monitor_result( x );
        }
        catch ( RS485::ErrFatal& e ) {
            ErrlogSingl::instance().write( ERR_EXCHANGE, e.what() );
        }
        catch (const std::exception& e ) {
            ErrlogSingl::instance().write( ERR_EXC_STAT, "Monitor thread finished!" );
            return 0;
        }
    }
    ErrlogSingl::instance().write( "Monitor thread finished!" );
    return 0;
}



