#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <pthread.h>
//#include <signal.h>

#include <errlog.h>
#include <astrotime.h>

#include "sensors.h"
#include "defnames.h"

Sensors_CMD Sensors::cmd = { std::string(program_cfg), false, false, false };
Sensors* Sensors::self_ptr = 0;
const int Sensors::initLifeSec = 4;
const int Sensors::parkLifeSec = 4;
pthread_t Sensors::thread_id;
pthread_mutex_t Sensors::Lock::mtx = PTHREAD_MUTEX_INITIALIZER;

Sensors::Sensors(): Sensorscfg( cmd.cfgfile ), deviceSensors( Sensorscfg, cmd.test_mode ),
    meteoData( Sensorscfg ), meteoScan( Sensorscfg, deviceSensors, meteoData ) {
    the = &meteoScan;
//    std::cerr << "Constructor Sensors\n";
}

void Sensors::create() {
    Lock lock;
    ErrlogSingl::instance().reopen();
    if(self_ptr == 0) self_ptr = new Sensors();
}

// void Sensors::destroy() {
//     if(self_ptr == 0) return;
//     Sensors* copy_ptr = self_ptr;       // To prevent commands execution
//     self_ptr = 0;                       // during Sensor destruction
//     delete copy_ptr;
// }

void Sensors::destroy() {
    Lock lock;       // To prevent commands execution during Dome destruction
    if(self_ptr == 0) return;
    delete self_ptr;
    self_ptr = 0;
}

Sensors* Sensors::device_ptr() {
    Lock lock;
    if( self_ptr == 0 ) throw ErrPark(ERR_INI_OBJECT, "Sensors object");
    return self_ptr;
}

int Sensors::specify( const std::vector<std::string>& opts ) {
    throw ErrSyn( ERR_PROT_WRCOMM, "run" );
    return 1;
}

int  Sensors::stop( const std::vector<std::string>& opts ) {
    throw ErrSyn( ERR_PROT_WRCOMM, "stop" );
    return 1;
}

const int check_period = 30;

bool Sensors::works() {
    static AstroDateTime tmp;
    static double prevs = 0;
    if(  exist() && tmp.currentSec() - prevs > check_period ){
        if( !self_ptr->deviceSensors.monitor_watch() ) {
            ErrlogSingl::instance().write( "Monitor thread doesn't work!" );
            return false;
        }
        prevs = tmp.currentSec();
    }
    return true;
}


typedef void(Sensors::*SETF_T)(const std::string&);
typedef std::pair<const std::string, SETF_T> SET_PAIR;

void Sensors::set(const std::string& key, const std::string& val) {
    throw ErrSyn( ERR_PROT_WRCOMM, "set" );
/*    static SET_PAIR set_kwd[] =  {
        SET_PAIR ("dome"  , &Dome::setAction),
        SET_PAIR ("limit" , &Dome::setLimit),
        SET_PAIR ("power" , &Dome::setPower)
    };
    static std::map<const std::string, SETF_T, Cmpi> setf(set_kwd, set_kwd + sizeof(set_kwd)/sizeof(set_kwd[0]));
    if( setf.find(key) == setf.end()) throw ErrSyn( ERR_PROT_WRKWD, val);
    (this->*setf[key])(val);*/
}

typedef std::string(Sensors::*GETF_T)();
typedef std::pair<const std::string, GETF_T> GET_PAIR;

std::string Sensors::get(const std::string& key) {
    static GET_PAIR get_kwd[] =  {
        GET_PAIR ("meteodata",&Sensors::data ),
        GET_PAIR ("support",  &Sensors::wind_gen ),
        GET_PAIR ("wind",     &Sensors::get_wind ),
        GET_PAIR ("sky",      &Sensors::get_sky ),
        GET_PAIR ("power",    &Sensors::get_voltage )
    };
    static std::map<const std::string, GETF_T, Cmpi> getf(get_kwd, get_kwd + sizeof(get_kwd)/sizeof(get_kwd[0]));
    if( getf.find(key) == getf.end()) throw ErrSyn( ERR_PROT_WRKWD, key);
    return (this->*getf[key])();
}

std::string Sensors::data() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str << "TEMP_EX=" << it.at(2) << " TEMP_IN=" << it.at(3)
        << " WIND=" << it.at(4) << " WIND_MAX=" << it.at(5) << " WIND_DIR=" << it.at(6)
        << " RH=" << it.at(7) << " PRESS=" << it.at(8) << " DEW_P=" << it.at(9);
    return str.str();
}

std::string Sensors::get_wind() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str << "WIND=" << it.at(4) << " WIND_MAX=" << it.at(5) << " WIND_DIR=" << it.at(6);
    return str.str();
}

std::string Sensors::get_sky() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str << "TEMP_AMB=" << it.at(16) << " TEMP_SEN=" << it.at(17) << " TEMP_SKY=" << it.at(18) << " SKY=" << it.at(19);
    return str.str();
}

std::string Sensors::wind_gen() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Support" );
    std::ostringstream str;
    it.at(14) = ( it.at(14) == "true" ) ? "on" : "off" ;
    it.at(15) = ( it.at(15) == "true" ) ? "on" : "off" ;
    str << "GEN_RPS=" << it.at(10) << " V_ACC=" << it.at(11)
        << " TEMP_BOX=" << it.at(12) << " V_BOX=" << it.at(13)
        << " SW1=" << it.at(14) << " SW2=" << it.at(15);
    return str.str();
}

std::string Sensors::get_voltage() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Wind" );
    std::ostringstream str;
    str << "V_ACC=" << it.at(11) << " V_BOX=" << it.at(13);
    return str.str();
}

