#ifndef _CLIENT_H
#define _CLIENT_H
#include <map>
#include <memory>
#include <server.h>
#include <configs.h>
#include <errlog.h>

class ClientHandler {
protected:
    std::auto_ptr<Client> client;
    ClientSocket::AddrType ips;
    unsigned int id_numb;
    bool success;
    std::map< std::string, std::string, Cmpi > answer_vals;
    static int start_id;
    static const int max_reconn = 5;
    int reconn_count;
public:
    ClientHandler( ClientSocket::AddrType );
    virtual ~ClientHandler(){ start_id -= 1000; client->close(); }
protected:
    virtual void rfunc( std::string );
    virtual void gfunc( std::string );
    virtual void sfunc( std::string );
    virtual void stopf( std::string );
    std::string id() {
        std::ostringstream ods;
        ods << id_numb;
        return ods.str();
    }
    int parse_structure( std::string& );
    void reconnect();
public:
    bool out(  const std::string& );
    bool command( const std::string& );
    void nonblocked( const std::string& );
    bool wait_command();
    bool wait_nonbusy( const std::string& );
    template<class T> T get_value( const std::string& key ) {
        T ret;
        if( answer_vals.find( key ) == answer_vals.end() )
            throw ErrFat( ERR_ALG, "Wrong value key" );
        std::istringstream ids( answer_vals[key] );
        ids >> ret;
        return ret;
    }
    std::string get_string( const std::string& key ) {
        if( answer_vals.find( key ) == answer_vals.end() )
            throw ErrFat( ERR_ALG, "Wrong value key" );
        return answer_vals[key];
    }
    bool get_status( const std::string& what ) {
        command("get status");
        return status( what );
    }
    bool status( const std::string& what ) {
        if( answer_vals.find( "status" ) != answer_vals.end() &&
            answer_vals["status"] == what ) return true;
        return false;
    }
};

#endif
