#ifndef __LINUX_SOCKET_H_
#define __LINUX_SOCKET_H_

/*
*   socket/linux.h is a part of programm carbon
*   $Id: linux.h,v 1.2 2007/12/18 20:02:21 victor Exp $
*   Copyright (C) 2005-2006  Matwey V. Kornilov
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include <list>
#include <stack>
#include <stdexcept>

#include "socket.h"

template <class Addr> class LinuxSocketTraits;

IP4Addr& operator<<( IP4Addr& addr, sockaddr_in* sa_addr );
const IP4Addr& operator>>( const IP4Addr& addr, sockaddr_in* sa_addr );


template <class Addr> class LinuxSocketTraits{
    typedef BasicSocketBase < LinuxSocketTraits< Addr > > SocketBase;
    typedef BasicClientSocket < LinuxSocketTraits< Addr > > ClientSocket;
    typedef BasicServerSocket < LinuxSocketTraits< Addr > > ServerSocket;
    typedef BasicSocketWatcher < LinuxSocketTraits< Addr > > SocketWatcher;
    typedef BasicSocketError < LinuxSocketTraits< Addr > > SocketError;
public:
    typedef Addr AddrType;

    enum Error{
        Undefined         = 0,
        ConnectionRefused = ECONNREFUSED,
        WouldBlock        = EWOULDBLOCK,
        NotSocket         = ENOTSOCK,
        ConnectionReset   = ECONNRESET,
        Acces             = EACCES,
        Permission        = EPERM,
        AddrInUse         = EADDRINUSE,
        InvalidParam      = EINVAL,
        InProgress        = EINPROGRESS,
        Again             = EAGAIN,
        Closed            = -1
    };

    class Watcher {
    private:
        std::list<ClientSocket*> m_clients;
        std::list<ServerSocket*> m_servers;
//      std::stack<ClientSocket*> m_clientsToRemove;
//      std::stack<ServerSocket*> m_serversToRemove;
        fd_set m_rfds, m_wfds, m_efds;
        int m_maxHandle;
    private:
/*
        template<class T> static inline void flushDeadSockets(std::list<T*>& list, std::stack<T*>& toRemove) {
            while(toRemove.size()) {
                list.remove( toRemove.top() );
                toRemove.pop();
            }
        }
*/
    public:
        void addSocket( ClientSocket* client );
        void addSocket( ServerSocket* server );
        void removeSocket( ClientSocket* client );
        void removeSocket( ServerSocket* server );
        void doPoll( unsigned int time );
    };

private:
    int m_handle;
public:
    LinuxSocketTraits();
    LinuxSocketTraits( int handle );
    ~LinuxSocketTraits();

    LinuxSocketTraits* accept();
    void bind( const Addr& addr );
    void connect( const Addr& addr );
    Addr getPeerName() const;
    Addr getSockName() const;

    void listen( int backlog );
    std::string read( std::size_t length );
    std::string read();
    std::size_t write( const std::string& str, std::size_t length );
    std::size_t write( const std::string& str );
    size_t bytesAvailable() const;

    void close();
};

typedef class LinuxSocketTraits<IP4Addr>          DefinedSocketTraits;

#endif // __LINUX_SOCKET_H_

