#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>

#include  <vector>
#include  <iostream>
#include  <iomanip>
#include <stdexcept>

#include "skys.h"

const double Czero = 273.15;
const std::string   sect_device("Devices");

const AData SkySens::GET_PRIM_DATA = 0xA0;
const int RAIN = 0x20;

SkySens::SkySens( Config& cfg, RS485& r, const std::string& s ): Module( cfg, r, s ) {
                                    //  
    urefer = cfg.inp<double>(sect_device, s, "Uref");
    amb.c1 = cfg.inp<double>(sect_device, s, "AmbC1");
    amb.c2 = cfg.inp<double>(sect_device, s, "AmbC2");
    amb.c3 = cfg.inp<double>(sect_device, s, "AmbC3");
    amb.c4 = cfg.inp<double>(sect_device, s, "AmbC4");

    sens.c1 = cfg.inp<double>(sect_device, s, "SensC1");
    sens.c2 = cfg.inp<double>(sect_device, s, "SensC2");
    sens.c3 = cfg.inp<double>(sect_device, s, "SensC3");
    sens.c4 = cfg.inp<double>(sect_device, s, "SensC4");

    sky.c1 = cfg.inp<double>(sect_device, s, "SkyC1");
    sky.c2 = cfg.inp<double>(sect_device, s, "SkyC2");
}

SkySens::~SkySens(){
//    std::cerr << "Destructor SkySens" << std::endl;
}

void SkySens::getData( MonitorInfo& c ) {
    std::vector<unsigned char> x = askData( GET_PRIM_DATA );
    if( x.size() == 8 ) {
        double u_amb = urefer/4096.0*(double)*(short*)&x[0];
        double r_amb = (u_amb*amb.c1)/(urefer - u_amb);
        c.skys_amb = 1.0/ ( (log(r_amb/amb.c2)/amb.c3) + (1.0/amb.c4) );

        double u_sens = urefer/4096.0*(double)*(short*)&x[4];
        double r_sens = (u_sens*sens.c1)/(urefer - u_sens);
        c.skys_sens = 1.0 / ( (log(r_sens/sens.c2)/sens.c3) + (1.0/sens.c4) );

        double u_sky = urefer/4096.0*(double)*(short*)&x[6] - urefer;
//        double dT = sky.c1*(double)*(short*)&x[6] + sky.c2;
        c.skys_sky = pow( u_sky*pow( sky.c1, sky.c2) + pow( c.skys_sens, sky.c2 ), 1.0/sky.c2 ) - Czero;
        c.skys_sens -= Czero;
        c.skys_amb -= Czero;
    }
    c.skys_stat = RAIN&getStatus()? 4 : ( (c.skys_sky - c.skys_amb > -10)? 3 : ( (c.skys_sky - c.skys_amb > -20)? 2 : 1) );
}

void SkySens::accumulate( std::vector<MonitorInfo>& v, MonitorInfo& acc ) {
    means( v, acc );
}

void SkySens::means( std::vector<MonitorInfo>& v, MonitorInfo& acc ) {
    unsigned int n = v.size();
    for( unsigned int i = 0; i < n; i++ ) {
        acc.skys_amb += v[i].skys_amb/n;
        acc.skys_sens += v[i].skys_sens/n;
        acc.skys_sky += v[i].skys_sky/n;
        acc.skys_stat += v[i].skys_stat;
    }
    acc.skys_stat /= n;
}


